# -*- coding: utf-8 -*-
# Part of Odoo. See LICENSE file for full copyright and licensing details.
from __future__ import absolute_import

import os
try:
    from messages import Msg
except ImportError:
    from odoo_env.messages import Msg
import subprocess

msg = Msg()


class Command:
    def __init__(
        self, parent, command=False, usr_msg=False, args=False,
        client_name=False):
        """
        :param parent: El objeto OdooEnv que lo contiene por los parametros
        :param command: El comando a ejecutar en el shell
        :param usr_msg: El mensaje a mostrarle al usuario
        :param args: Argumentos para chequear, define si se ejecuta o no
        :return: El objeto Comando que se ejecutara luego
        """
        self._parent = parent
        self._command = command
        self._usr_msg = usr_msg
        self._args = args
        self._client_name = client_name

    def check(self):
        # si no tiene argumentos para chequear no requiere chequeo,
        # lo dejamos pasar
        if not self._args:
            return True

        # le pasamos el chequeo al objeto especifico
        return self.check_args()

    def execute(self):
        cmd = self.command
        self.subrpocess_call(cmd)

    def subrpocess_call(self, params, shell=False):
        """ Run command or command list with arguments.  Wait for commands to
            complete
            If args.verbose is true, prints command
            If any errors stop list execution and returns error
            if shell=True go shell mode (only for --cron-jobs)

        :param params: command or command list
        :return: error return
        """
        # if not a list convert to a one element list
        params = params if isinstance(params, list) else [params]

        # traverse list executing shell commands
        for _cmd in params:
            # if shell = True we do no split
            cmd = _cmd if shell else _cmd.split()
            if self._parent.verbose:
                msg.run(' ')
                if shell:
                    msg.run(cmd)
                else:
                    msg.run(' '.join(cmd))
                msg.run(' ')
            ret = subprocess.call(cmd, shell=shell)
            if ret:
                return msg.err('The command {} returned with {}'.format(
                    cmd,
                    str(ret)))

    @property
    def args(self):
        return self._args

    @property
    def usr_msg(self):
        return self._usr_msg

    @property
    def command(self):
        return self._command


class CreateGitignore(Command):
    def execute(self):
        # crear el gitignore en el archivo que viene del comando
        value = '.idea/'
        with open(self._command, 'w') as f:
            f.write(value)


class MakedirCommand(Command):
    def check_args(self):
        # si el directorio existe no lo creamos
        return not os.path.isdir(self._args)


class ExtractSourcesCommand(Command):
    def check_args(self):
        return True


class CloneRepo(Command):
    def check_args(self):
        # si el directorio no existe dejamos clonar
        return not os.path.isdir(self._args)


class PullRepo(Command):
    def check_args(self):
        # si el directorio existe dejamos pulear
        return os.path.isdir(self._args)


class PullImage(Command):
    @staticmethod
    def check_args():
        return True


class CreateNginxTemplate(Command):
    def check_args(self):
        # si el archivo existe no lo dejamos pasar
        return not os.path.isfile(self._args)

    def execute(self):
        # crear el nginx.conf
        with open('/usr/local/nginx.conf', 'r') as f:
            conf = f.read()

        # poner el nombre del cliente en el config
        conf = conf.replace('$client$', self._client_name)

        with open(self._command, 'w') as f:
            f.write(conf)


class MessageOnly(Command):
    @staticmethod
    def check_args(self):
        """ Siempre lo dejamos pasar
        """
        return True

    @staticmethod
    def execute():
        pass



# -*- coding: utf-8 -*-

BASE_DIR = '/odoo_ar/'
IN_CONFIG = '/opt/odoo/etc/'
IN_DATA = '/opt/odoo/data'
IN_LOG = '/var/log/odoo'
IN_CUSTOM_ADDONS = '/opt/odoo/custom-addons'
IN_EXTRA_ADDONS = '/opt/odoo/extra-addons'
IN_DIST_PACKAGES = '/usr/lib/python{}/dist-packages'
IN_DIST_LOCAL_PACKAGES = '/usr/local/lib/python{}/dist-packages'
IN_BACKUP_DIR = '/var/odoo/backups/'


# -*- coding: utf-8 -*-
# Part of Odoo. See LICENSE file for full copyright and licensing details.

import os
try:
    from messages import Msg
    from constants import BASE_DIR
    from repos import Repo
    from images import Image
except ImportError:
    from odoo_env.messages import Msg
    from odoo_env.constants import BASE_DIR
    from odoo_env.repos import Repo
    from odoo_env.images import Image

msg = Msg()


class Client(object):
    def __init__(self, odooenv, name):
        """ Busca el cliente en la estructura de directorios, pero si no lo
            encuentra pide un directorio donde esta el repo que lo contiene
        """
        # parent es siempre un objeto OdooEnv
        self._parent = odooenv
        self._name = name

        # si estamos en test accedo a data
        if name[0:5] == 'test_':
            path = os.path.dirname(os.path.abspath(__file__))
            path = path.replace('odoo_env', 'odoo_env/data')
            manifest = self.get_manifest(path)
        else:
            manifest = self.get_manifest(BASE_DIR)
        if not manifest:
            msg.inf('Can not find client {} in this host. Please provide path '
                    'to repo\n where it is or hit Enter to exit.'
                    '\n'.format(self._name))

            # mantener compatibilidad con python2
            import six
            path = six.moves.input('path = ')
            manifest = self.get_manifest(path)
            if not manifest:
                msg.err('Can not find client {} in this host'.format(name))

            msg.inf('Client found!')
            msg.inf('Name {}\nversion {}\n'.format(manifest.get('name'),
                                                   manifest.get('version')))

        # Chequar que este todo bien
        if not manifest.get('docker'):
            msg.err('No images in manifest {}'.format(self.name))

        if not manifest.get('repos'):
            msg.err('No repos in manifest {}'.format(self.name))

        self._port = manifest.get('port')
        if not self._port:
            msg.err('No port in manifest {}'.format(self.name))

        ver = manifest.get('version')
        if not ver:
            msg.err('No version tag in manifest {}'.format(self.name))
        x = ver.find('.') + 1
        y = ver[x:].find('.') + x
        self._version = ver[0:y]

        # Crear imagenes y repos
        self._repos = []
        for rep in manifest.get('repos'):
            self._repos.append(Repo(rep))

        self._images = []
        for img in manifest.get('docker'):
            self._images.append(Image(img))

        # todo codigo repetido
        # get first word of name in lowercase
        name = manifest.get('name').lower()
        if not self._name == name.split()[0]:
            msg.err('You intend to install client {} but in manifest, '
                    'the name is {}'.format(self._name, manifest.get('name')))

    def get_manifest(self, path):
        """
        :param path: base dir to walk searching for manifest
        :return: parsed manifest file as dictionary
        """
        for root, dirs, files in os.walk(path):
            for file in ['__openerp__.py', '__manifest__.py']:
                if file in files:
                    manifest_file = '{}/{}'.format(root, file)
                    manifest = self.load_manifest(manifest_file)
                    # todo codigo repetido
                    # get first word of name in lowercase
                    name = manifest.get('name').lower()
                    # por si viene sin name
                    if name:
                        name = name.split()[0]
                        if name == self._name:
                            return manifest

        return False

    @staticmethod
    def load_manifest(filename):
        """
        Loads a manifest
        :param filename: absolute filename to manifest
        :return: manifest in dictionary format
        """
        manifest = ''
        with open(filename, 'r') as f:
            for line in f:
                if line.strip() and line.strip()[0] != '#':
                    manifest += line
            try:
                ret = eval(manifest)
            except Exception:
                return {'name': 'none'}
            return ret

    def image(self, image_name):
        for img_dict in self._images:
            if img_dict.get('name') == image_name:
                img = img_dict.get('img')
                ver = img_dict.get('ver')
                ret = img_dict.get('usr')
                if img:
                    ret += '/' + img
                if ver:
                    ret += ':' + ver
                return ret
        msg.err('There is no {} image found in this manifest'.format(
            image_name))

    def get_image(self, value):
        for image in self._images:
            if image.short_name == value:
                return image
        return False

    @property
    def name(self):
        return self._name

    @property
    def version(self):
        return self._version

    @property
    def numeric_ver(self):
        return float(self.version[0:2])

    @property
    def repos(self):
        return self._repos

    @property
    def images(self):
        return self._images

    @property
    def port(self):
        return self._port

    @property
    def psql_dir(self):
        return self.base_dir + 'postgresql/'

    @property
    def base_dir(self):
        return '{}odoo-{}/{}/'.format(BASE_DIR, self._version, self._name)

    @property
    def version_dir(self):
        return '{}odoo-{}/'.format(BASE_DIR, self._version)

    @property
    def sources_dir(self):
        """ links to repos for this client only pointing to sources_com """
        return self.base_dir + 'sources/'

    @property
    def sources_com(self):
        """ real repos for this odoo Version, all clients """
        return '{}odoo-{}/sources/'.format(BASE_DIR, self._version)

    @property
    def nginx_dir(self):
        """ Base dir for nginx """
        return '{}nginx/'.format(BASE_DIR)

    @property
    def backup_dir(self):
        return self.base_dir + 'backup_dir/'



#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
###############################################
# shortcut for sudo docker
# sudo docker $*
###############################################
from __future__ import print_function

import subprocess
import sys


def process_input(params):
    # si no tiene parametros termino
    if len(params) <= 1:
        return []

    # el primer elemento es el nombre de este archivo, lo saco
    params.pop(0)

    # agregamos sudo docker al principio
    params[0:0] = ['sudo', 'docker']

    # pseudo sintaxis

    if params[2] == '-h':
        print('Help for sd')
        print('sd               - short for sudo docker')
        print('sd inside image  - open console inside image')
        print('sd rmall         - remove all images in memory')
        print('sd rmdiskall     - remove all images in disk')
        print('sd rmuntag       - remove all untagged images in disk')
        print('sd attach name   - attach to a running container by name')
        print(' ')
        exit()

    if params[2] == 'inside':
        try:
            print('going inside image ' + params[3])
            params[2:3] = ['run', '-it', '--rm', '--entrypoint=/bin/bash']
        except Exception:
            params = []

    if params[2] == 'rmall':
        try:
            print('removing all images in memory')
            params[2:3] = ['rm', '-f', '$(sudo docker ps -a -q)']
        except Exception:
            params = []

    if params[2] == 'rmdiskall':
        try:
            print('removing all images in disk')
            params[2:3] = ['rmi', '$(sudo docker images -q)']
        except Exception:
            params = []

    if params[2] == 'rmuntag':
        try:
            print('removing all untagged images in disk')
            cmd = '$(sudo docker images | grep "^<none>" | awk "{print $3}")'
            params[2:3] = ['rmi', cmd]
        except Exception:
            params = []

    if params[2] == 'attach':
        try:
            container_name = params[3]
            print('attaching to ' + container_name)

            # sd exec -it mario bash

            params[2:3] = ['exec', '-it']
            params[4:5] = [container_name, 'bash']
        except Exception:
            params = []

    params = ' '.join(params)
    return params


def main():
    params = process_input(sys.argv)
    try:
        if len(params) > 1:
            exit(subprocess.call(params, shell=True))
    except Exception as ex:
        print(ex)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from odoo_env.messages import Msg


def get_param(args, param):
    if param == 'client':
        if args.client:
            return args.client[0]
        else:
            Msg().err('Need -c option (client name). Process aborted')

    if param == 'database':
        if args.database:
            return args.database[0]
        else:
            database = args.client[0] + '_prod'
            Msg().inf('Using default database: %s, use -d to '
                      'specify database' % database)
            return database

    if param == 'module':
        if args.module:
            return args.module
        else:
            Msg().inf('Updaging all modules. Use -m to specify single module '
                      'or a comma separated list of modules.')
            return ['all']

    if param == 'backup_file':
        if args.backup_file:
            return args.backup_file[0]
        else:
            Msg().inf('Restoring newest backup. Use -f to store specific one.')
            return False

    if param == 'deactivate':
        if args.deactivate:
            return args.deactivate
        else:
            return False



# -*- coding: utf-8 -*-

import setuptools
from odoo_env import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='odoo-env',
    version=__version__,
    author='Jorge E. Obiols',
    description='An environment to manage Docker for Odoo',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/jobiols/odoo-env',
    author_email='jorge.obiols@gmail.com',
    entry_points={
        'console_scripts': [
            'oe=odoo_env.oe:main',
            'sd=odoo_env.sd:main'
        ],
    },
    install_requires=[],
    data_files=[('', ['odoo_env/data/nginx.conf'])],
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Testing :: Unit",
        "Topic :: System :: Software Distribution",
    ],
    keywords="odoo docker environment",
)
